@@ -1,8 +1,8 @@
 /* cmdline.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002-2013 David Anderson
- * Copyright (C) 2002-2013 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2014 David Anderson
+ * Copyright (C) 2002-2014 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ static void restore_sanity(void);
 static void restore_ifile_sanity(void);
 static int pseudo_command(char *);
 static void check_special_handling(char *);
+static int is_executable_in_PATH(char *);
 static int is_shell_script(char *);
 static void list_aliases(char *);
 static int allocate_alias(int);
@@ -195,6 +196,29 @@ check_special_handling(char *s)
         }
 }
 
+static int
+is_executable_in_PATH(char *filename)
+{
+	char buf1[BUFSIZE];
+	char buf2[BUFSIZE];
+	char *tok, *path;
+
+        if ((path = getenv("PATH")))
+		strcpy(buf2, path);
+	else
+		return FALSE;
+
+	tok = strtok(buf2, ":");
+	while (tok) {
+		sprintf(buf1, "%s/%s", tok, filename);
+		if (file_exists(buf1, NULL) && 
+		    (access(buf1, X_OK) == 0))
+			return TRUE;
+		tok = strtok(NULL, ":");
+	}
+
+	return FALSE;
+}
 
 /*
  *  At this point the only pseudo commands are the "r" (repeat) and 
@@ -203,9 +227,11 @@ check_special_handling(char *s)
  *    1. an "r" alone, or "!!" along, just means repeat the last command.
  *    2. an "r" followed by a number, means repeat that command from the
  *       history table.
- *    3. an "r" followed by one or more non-decimal characters means to
+ *    3. an "!" followed by a number that is not the name of a command 
+ *       in the user's PATH, means repeat that command from the history table.
+ *    4. an "r" followed by one or more non-decimal characters means to
  *       seek back until a line-beginning match is found. 
- *    4. an "h" alone, or a string beginning with "hi", means history.
+ *    5. an "h" alone, or a string beginning with "hi", means history.
  */
 static int
 pseudo_command(char *input)
@@ -242,6 +268,11 @@ pseudo_command(char *input)
                 goto rerun;
         }
 
+        if ((input[0] == '!') && decimal(&input[1], 0) &&
+	    !is_executable_in_PATH(first_nonspace(&input[1]))) {
+		p = first_nonspace(&input[1]);
+		goto rerun;
+	}
 
 	if (STRNEQ(input, "r ")) {
                 if (!history_offset)
@@ -912,7 +943,7 @@ set_command_prompt(char *new_prompt)
  *  Signal number 0 is sent for a generic restart.
  */
 #define MAX_RECURSIVE_SIGNALS (10)
-#define MAX_SIGINTS_ACCEPTED  (3)
+#define MAX_SIGINTS_ACCEPTED  (1)
 
 void
 restart(int sig)
@@ -956,6 +987,10 @@ restart(int sig)
 		pc->flags &= ~IN_RESTART;
 		if (pc->sigint_cnt == MAX_SIGINTS_ACCEPTED) {
 			restore_sanity();
+			if (pc->ifile_in_progress) {
+				pc->ifile_in_progress = 0;
+				pc->ifile_offset = 0;
+			}
 			break;
 		}
 		return;
@@ -968,7 +1003,7 @@ restart(int sig)
 		break;
 	}
 
-	fprintf(fp, "\n");
+	fprintf(stderr, "\n");
 
 	pc->flags &= ~(IN_FOREACH|IN_GDB|IN_RESTART);
 	longjmp(pc->main_loop_env, 1);
@@ -1405,6 +1440,9 @@ exec_input_file(void)
                 }
 
                 exec_command();
+
+		if (received_SIGINT())
+			goto done_input;
         }
 
 done_input:
@@ -2430,6 +2468,8 @@ exec_args_input_file(struct command_table_entry *ct, struct args_input_file *aif
 	char *new_args[MAXARGS];
 	char *orig_args[MAXARGS];
 	char orig_line[BUFSIZE];
+	char *save_args[MAXARGS];
+	char save_line[BUFSIZE];
 
 	if ((pc->args_ifile = fopen(aif->fileptr, "r")) == NULL)
 		error(FATAL, "%s: %s\n", aif->fileptr, strerror(errno));
@@ -2440,11 +2480,24 @@ exec_args_input_file(struct command_table_entry *ct, struct args_input_file *aif
 	BCOPY(args, orig_args, sizeof(args));
 	orig_argcnt = argcnt;
 
+	/*
+	 *  Commands cannot be trusted to leave the arguments intact.
+	 *  Stash them here and restore them each time through the loop.
+	 */
+	save_args[0] = save_line;
+	for (i = 0; i < orig_argcnt; i++) {
+		strcpy(save_args[i], orig_args[i]);
+		save_args[i+1] = save_args[i] + strlen(save_args[i]) + 2;
+	}
+
 	while (fgets(buf, BUFSIZE-1, pc->args_ifile)) {
 		clean_line(buf);
 		if ((strlen(buf) == 0) || (buf[0] == '#'))
 			continue;		
 
+		for (i = 1; i < orig_argcnt; i++)
+			strcpy(orig_args[i], save_args[i]);
+
 		if (aif->is_gdb_cmd) {
 			console("(gdb) before: [%s]\n", orig_line);
 			strcpy(pc->orig_line, orig_line);
@@ -2501,6 +2554,9 @@ exec_args_input_file(struct command_table_entry *ct, struct args_input_file *aif
 			pc->cmd_cleanup(pc->cmd_cleanup_arg);
 
 		free_all_bufs();
+
+		if (received_SIGINT())
+			break;		
 	}
 
 	fclose(pc->args_ifile);
