@@ -7,8 +7,8 @@
  * netdump dumpfiles, the facilities in netdump.c are used.  For
  * compressed dumpfiles, the facilities in this file are used.
  *
- * Copyright (C) 2004-2013 David Anderson
- * Copyright (C) 2004-2013 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2004-2014 David Anderson
+ * Copyright (C) 2004-2014 Red Hat, Inc. All rights reserved.
  * Copyright (C) 2005  FUJITSU LIMITED
  * Copyright (C) 2005  NEC Corporation
  *
@@ -119,7 +119,7 @@ map_cpus_to_prstatus_kdump_cmprs(void)
 	nrcpus = (kt->kernel_NR_CPUS ? kt->kernel_NR_CPUS : NR_CPUS);
 
 	for (i = 0, j = 0; i < nrcpus; i++) {
-		if (in_cpu_map(ONLINE, i)) {
+		if (in_cpu_map(ONLINE_MAP, i)) {
 			dd->nt_prstatus_percpu[i] = nt_ptr[j++];
 			dd->num_prstatus_notes = 
 				MAX(dd->num_prstatus_notes, i+1);
@@ -618,7 +618,7 @@ restart:
 		len = bitmap_len;
 		while (len) {
 			bytes_read = read(dd->dfd, bufptr, len);
-			if (bytes_read  < 0) {
+			if (bytes_read <= 0) {
 				error(INFO, "%s: cannot read memory bitmap\n",
 					DISKDUMP_VALID() ? "diskdump"
 					: "compressed kdump");
@@ -838,11 +838,13 @@ is_diskdump(char *file)
 	dd->flags |= SNAPPY_SUPPORTED;
 #endif
 
+	pc->read_vmcoreinfo = vmcoreinfo_read_string;
+
 	if ((pc->flags2 & GET_LOG) && KDUMP_CMPRS_VALID()) {
 		pc->dfd = dd->dfd;
 		pc->readmem = read_diskdump;
 		pc->flags |= DISKDUMP;
-		get_log_from_vmcoreinfo(file, vmcoreinfo_read_string);
+		get_log_from_vmcoreinfo(file);
 	}
 
 	return TRUE;
@@ -1043,7 +1045,7 @@ cache_page(physaddr_t paddr)
 
 #ifdef SNAPPY
 		ret = snappy_uncompressed_length((char *)dd->compressed_page,
-						 pd.size, &retlen);
+						 pd.size, (size_t *)&retlen);
 		if (ret != SNAPPY_OK) {
 			error(INFO, "%s: uncompress failed: %d\n",
 			      DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
@@ -1053,7 +1055,7 @@ cache_page(physaddr_t paddr)
 
 		ret = snappy_uncompress((char *)dd->compressed_page, pd.size,
 					(char *)dd->page_cache_hdr[i].pg_bufptr,
-					&retlen);
+					(size_t *)&retlen);
 		if ((ret != SNAPPY_OK) || (retlen != block_size)) {
 			error(INFO, "%s: uncompress failed: %d\n", 
 			      DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
@@ -1191,7 +1193,7 @@ get_diskdump_panic_task(void)
 		if (kernel_symbol_exists("crashing_cpu") &&
 		    cpu_map_addr("online")) {
 			get_symbol_data("crashing_cpu", sizeof(int), &i);
-			if ((i >= 0) && in_cpu_map(ONLINE, i)) {
+			if ((i >= 0) && in_cpu_map(ONLINE_MAP, i)) {
 				if (CRASHDEBUG(1))
 					error(INFO, "get_diskdump_panic_task: "
 					    "active_set[%d]: %lx\n", 
@@ -1558,30 +1560,23 @@ __diskdump_memory_dump(FILE *fp)
 	fprintf(fp, "                tv_sec: %lx\n", dh->timestamp.tv_sec);
 	fprintf(fp, "               tv_usec: %lx\n", dh->timestamp.tv_usec);
 	fprintf(fp, "              status: %x (", dh->status);
-        others = 0;
 	switch (dd->flags & (DISKDUMP_LOCAL|KDUMP_CMPRS_LOCAL))
 	{
         case DISKDUMP_LOCAL:
 		if (dh->status == DUMP_HEADER_COMPLETED)
-			fprintf(fp, "%sDUMP_HEADER_COMPLETED", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_HEADER_COMPLETED");
 		else if (dh->status == DUMP_HEADER_INCOMPLETED)
-			fprintf(fp, "%sDUMP_HEADER_INCOMPLETED", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_HEADER_INCOMPLETED");
 		else if (dh->status == DUMP_HEADER_COMPRESSED)
-			fprintf(fp, "%sDUMP_HEADER_COMPRESSED", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_HEADER_COMPRESSED");
 		break;
 	case KDUMP_CMPRS_LOCAL:
 		if (dh->status & DUMP_DH_COMPRESSED_ZLIB)
-			fprintf(fp, "%sDUMP_DH_COMPRESSED_ZLIB", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_DH_COMPRESSED_ZLIB");
 		if (dh->status & DUMP_DH_COMPRESSED_LZO)
-			fprintf(fp, "%sDUMP_DH_COMPRESSED_LZO", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_DH_COMPRESSED_LZO");
 		if (dh->status & DUMP_DH_COMPRESSED_SNAPPY)
-			fprintf(fp, "%sDUMP_DH_COMPRESSED_SNAPPY", 
-				others++ ? "|" : "");
+			fprintf(fp, "DUMP_DH_COMPRESSED_SNAPPY");
 		break;
 	}
 	fprintf(fp, ")\n");
@@ -1901,6 +1896,9 @@ vmcoreinfo_read_string(const char *key)
 	char keybuf[BUFSIZE];
 	const off_t failed = (off_t)-1;
 
+	if (dd->header->header_version < 3)
+		return NULL;
+
 	buf = value_string = NULL;
 	size_vmcoreinfo = dd->sub_header_kdump->size_vmcoreinfo;
 	offset = dd->sub_header_kdump->offset_vmcoreinfo;
@@ -1966,7 +1964,7 @@ diskdump_display_regs(int cpu, FILE *ofp)
 	char *user_regs;
 	size_t len;
 
-	if (cpu >= NR_CPUS || dd->nt_prstatus_percpu[cpu] == NULL) {
+	if (!diskdump_get_prstatus_percpu(cpu)) {
 		error(INFO, "registers not collected for cpu %d\n", cpu);
 		return;
 	}
@@ -2048,3 +2046,20 @@ diskdump_display_regs(int cpu, FILE *ofp)
 		);
 	}
 }
+
+void
+dump_registers_for_compressed_kdump(void)
+{
+	int c;
+
+	if (!KDUMP_CMPRS_VALID() || (dd->header->header_version < 4) ||
+	    !(machine_type("X86") || machine_type("X86_64") || machine_type("ARM64")))
+		error(FATAL, "-r option not supported for this dumpfile\n");
+
+	for (c = 0; c < kt->cpus; c++) {
+		fprintf(fp, "%sCPU %d:\n", c ? "\n" : "", c);
+		diskdump_display_regs(c, fp);
+	}
+}
+
+
